@@ -14,23 +14,27 @@ from . import helper
 from ..utils import misc
 
 def get_args():
-    parser = argparse.ArgumentParser()
+    parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
+
+    optional = parser._action_groups.pop()
+    required = parser.add_argument_group('required arguments')
 
     # Required arguments
-    parser.add_argument('--eventalign', dest='eventalign', help='Eventalign filepath from nanopolish.',required=True)
-    parser.add_argument('--summary', dest='summary', help='Summary filepath from nanopolish.',required=True)
-    parser.add_argument('--out_dir', dest='out_dir', help='Output directory.',required=True)
+    required.add_argument('--eventalign', dest='eventalign', help='eventalign filepath, the output from nanopolish.',required=True)
+    required.add_argument('--summary', dest='summary', help='eventalign summary filepath, the output from nanopolish.',required=True)
+    required.add_argument('--out_dir', dest='out_dir', help='output directory.',required=True)
     
-    parser.add_argument('--ensembl', dest='ensembl', help='ensembl version.',type=int, default=91)
+    required.add_argument('--ensembl', dest='ensembl', help='ensembl version for gene-transcript mapping.',type=int, default=91)
+    required.add_argument('--species', dest='species', help='species for ensembl gene-transcript mapping.', default='homo_sapiens')
 
     # Optional
     # parser.add_argument('--features', dest='features', help='Signal features to extract.',type=list,default=['norm_mean'])
-    parser.add_argument('--genome', dest='genome', help='.',default=False,action='store_true') 
-    parser.add_argument('--n_processes', dest='n_processes', help='Number of processes to run.',type=int, default=1)
-    parser.add_argument('--readcount_min', dest='readcount_min', help='Minimum of read counts per gene.',type=int, default=10)
-    parser.add_argument('--readcount_max', dest='readcount_max', help='Maximum of read counts per gene.',type=int, default=5000)
-    parser.add_argument('--resume', dest='resume', help='Resume.',default=False,action='store_true') #todo
+    optional.add_argument('--genome', dest='genome', help='to run on Genomic coordinates. Without this argument, the program will run on transcriptomic coordinates',default=False,action='store_true') 
+    optional.add_argument('--n_processes', dest='n_processes', help='number of processes to run.',type=int, default=1)
+    optional.add_argument('--readcount_max', dest='readcount_max', help='maximum read counts per gene.',type=int, default=1000)
+    optional.add_argument('--resume', dest='resume', help='resume from the previous run.',default=False,action='store_true') #todo
 
+    parser._action_groups.append(optional)
     return parser.parse_args()
 
 def combine(read_name,eventalign_per_read,out_paths,locks):
@@ -84,7 +88,7 @@ def combine(read_name,eventalign_per_read,out_paths,locks):
         f.write('%s\n' %(read_name))    
 
 
-def parallel_combine(eventalign_filepath,summary_filepath,out_dir,n_processes):
+def parallel_combine(eventalign_filepath,summary_filepath,out_dir,n_processes,resume):
     # Create output paths and locks.
     out_paths,locks = dict(),dict()
     for out_filetype in ['hdf5','log']:
@@ -92,9 +96,13 @@ def parallel_combine(eventalign_filepath,summary_filepath,out_dir,n_processes):
         locks[out_filetype] = multiprocessing.Lock()
         
         
-    # Create empty files.
-    open(out_paths['hdf5'],'w').close()
-    open(out_paths['log'],'w').close()
+    read_names_done = []
+    if resume and os.path.exists(out_paths['log']):
+        read_names_done = [line.rstrip('\n') for line in open(out_paths['log'],'r')]
+    else:
+        # Create empty files.
+        open(out_paths['hdf5'],'w').close()
+        open(out_paths['log'],'w').close()
 
     # Create communication queues.
     task_queue = multiprocessing.JoinableQueue(maxsize=n_processes * 2)
@@ -119,7 +127,8 @@ def parallel_combine(eventalign_filepath,summary_filepath,out_dir,n_processes):
                 eventalign_per_read += [row_eventalign]
             else: 
                 # Load a read info to the task queue.
-                task_queue.put((read_name,eventalign_per_read,out_paths))
+                if read_name not in read_names_done:
+                    task_queue.put((read_name,eventalign_per_read,out_paths))
                 # Next read.
                 try:
                     row_summary = next(reader_summary)
@@ -160,7 +169,7 @@ def t2g(gene_id,ensembl):
                 
     return tx_ids, t2g_dict
             
-def parallel_preprocess_gene(ensembl,out_dir,n_processes,readcount_min,readcount_max,resume):
+def parallel_preprocess_gene(ensembl,out_dir,n_processes,readcount_max,resume):
     
     # Create output paths and locks.
     out_paths,locks = dict(),dict()
@@ -172,7 +181,7 @@ def parallel_preprocess_gene(ensembl,out_dir,n_processes,readcount_min,readcount
     gene_ids_done = []
     if resume and os.path.exists(out_paths['index']):
         df_index = pd.read_csv(out_paths['index'],sep=',')
-        gene_ids_done = list(df_index['gene_id'].unique())
+        gene_ids_done = list(df_index['idx'].unique())
     else:
         # with open(out_paths['json'],'w') as f:
         #     f.write('{\n')
@@ -220,13 +229,16 @@ def parallel_preprocess_gene(ensembl,out_dir,n_processes,readcount_min,readcount
             for tx_id in tx_ids:
                 if tx_id not in f: # no eventalign for tx_id
                     continue
-                n_reads += len(f[tx_id])
-                if (n_reads >= readcount_min) and (n_reads <= readcount_max):
-                    for read_id in f[tx_id].keys():
-                        if read_id not in read_ids:
-                            data_dict[read_id] = f[tx_id][read_id]['events'][:]
-                            read_ids += [read_id]
-            if (len(read_ids) >= readcount_min) and (len(read_ids) <= readcount_max):
+                # n_reads += len(f[tx_id])                
+                for read_id in f[tx_id].keys():
+                    if (n_reads < readcount_max) and (read_id not in read_ids):
+                        data_dict[read_id] = f[tx_id][read_id]['events'][:]
+                        read_ids += [read_id]
+                        n_reads += 1
+                    elif n_reads >= readcount_max:
+                        break
+                    
+            if len(read_ids) > 0:
                 task_queue.put((gene_id,data_dict,t2g_mapping,out_paths)) # Blocked if necessary until a free slot is available. 
                 gene_ids_processed += [gene_id]
 
@@ -344,7 +356,7 @@ def preprocess_gene(gene_id,data_dict,t2g_mapping,out_paths,locks):
         f.write(log_str + '\n')
 
 
-def parallel_preprocess_tx(out_dir,n_processes,readcount_min,readcount_max,resume):
+def parallel_preprocess_tx(out_dir,n_processes,readcount_max,resume):
     
     # Create output paths and locks.
     out_paths,locks = dict(),dict()
@@ -380,13 +392,16 @@ def parallel_preprocess_tx(out_dir,n_processes,readcount_min,readcount_max,resum
             if resume and (tx_id in tx_ids_done):
                 continue
                 
-            n_reads = len(f[tx_id])
             read_ids = []
-            if (n_reads >= readcount_min) and (n_reads <= readcount_max):
-                data_dict = dict()
-                for read_id in f[tx_id].keys():
+            n_reads = 0
+            data_dict = dict()
+            for read_id in f[tx_id].keys():
+                if n_reads < readcount_max:
                     data_dict[read_id] = f[tx_id][read_id]['events'][:]
                     read_ids += [read_id]
+                    n_reads += 1
+                else:
+                    break
             task_queue.put((tx_id,data_dict,out_paths)) # Blocked if necessary until a free slot is available. 
             tx_ids_processed += [tx_id]
 
@@ -485,7 +500,7 @@ def main():
     summary_filepath = args.summary
     out_dir = args.out_dir
     ensembl_version = args.ensembl
-    readcount_min = args.readcount_min
+    ensembl_species = args.species
     readcount_max = args.readcount_max
     resume = args.resume
     genome = args.genome
@@ -496,15 +511,15 @@ def main():
     # (1) For each read, combine multiple events aligned to the same positions, the results from nanopolish eventalign, into a single event per position.
     eventalign_log_filepath = os.path.join(out_dir,'eventalign.log')
     if not helper.is_successful(eventalign_log_filepath):
-        parallel_combine(eventalign_filepath,summary_filepath,out_dir,n_processes)
+        parallel_combine(eventalign_filepath,summary_filepath,out_dir,n_processes,resume)
     
     # (2) Create a .json file, where the info of all reads are stored per position, for modelling.
     if genome:
-        ensembl = EnsemblRelease(ensembl_version) # human reference genome GRCh38 release 91 used in the ont mapping.    
-        parallel_preprocess_gene(ensembl,out_dir,n_processes,readcount_min,readcount_max,resume)
+        ensembl = EnsemblRelease(ensembl_version,ensembl_species) # Default: human reference genome GRCh38 release 91 used in the ont mapping.    
+        parallel_preprocess_gene(ensembl,out_dir,n_processes,readcount_max,resume)
 
     else:
-        parallel_preprocess_tx(out_dir,n_processes,readcount_min,readcount_max,resume)
+        parallel_preprocess_tx(out_dir,n_processes,readcount_max,resume)
 
 
 if __name__ == '__main__':
@@ -513,7 +528,6 @@ if __name__ == '__main__':
         --eventalign EVENTALIGN --summary SUMMARY --out_dir OUT_DIR \
                       [--ensembl ENSEMBL] [--genome] \
                       [--n_processes N_PROCESSES] \
-                      [--readcount_min READCOUNT_MIN] \
                       [--readcount_max READCOUNT_MAX] [--resume] \
     """
     main()
